@@ -1,7 +1,6 @@
 /* Output generating routines for GDB CLI.
 
-   Copyright (C) 1999, 2000, 2002, 2003, 2005, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 1999-2015 Free Software Foundation, Inc.
 
    Contributed by Cygnus Solutions.
    Written by Fernando Nasser for Cygnus.
@@ -24,97 +23,43 @@
 #include "defs.h"
 #include "ui-out.h"
 #include "cli-out.h"
-#include "gdb_string.h"
-#include "gdb_assert.h"
+#include "vec.h"
 
-struct ui_out_data
-  {
-    struct ui_file *stream;
-    struct ui_file *original_stream;
-    int suppress_output;
-  };
-typedef struct ui_out_data cli_out_data;
+typedef struct cli_ui_out_data cli_out_data;
 
-/* These are the CLI output functions */
-
-static void cli_table_begin (struct ui_out *uiout, int nbrofcols,
-			     int nr_rows, const char *tblid);
-static void cli_table_body (struct ui_out *uiout);
-static void cli_table_end (struct ui_out *uiout);
-static void cli_table_header (struct ui_out *uiout, int width,
-			      enum ui_align alig, const char *col_name,
-			      const char *colhdr);
-static void cli_begin (struct ui_out *uiout, enum ui_out_type type,
-		       int level, const char *lstid);
-static void cli_end (struct ui_out *uiout, enum ui_out_type type, int level);
-static void cli_field_int (struct ui_out *uiout, int fldno, int width,
-			   enum ui_align alig, const char *fldname, int value);
-static void cli_field_skip (struct ui_out *uiout, int fldno, int width,
-			    enum ui_align alig, const char *fldname);
-static void cli_field_string (struct ui_out *uiout, int fldno, int width,
-			      enum ui_align alig, const char *fldname,
-			      const char *string);
-static void cli_field_fmt (struct ui_out *uiout, int fldno,
-			   int width, enum ui_align align,
-			   const char *fldname, const char *format,
-			   va_list args) ATTR_FORMAT (printf, 6, 0);
-static void cli_spaces (struct ui_out *uiout, int numspaces);
-static void cli_text (struct ui_out *uiout, const char *string);
-static void cli_message (struct ui_out *uiout, int verbosity,
-			 const char *format, va_list args)
-     ATTR_FORMAT (printf, 3, 0);
-static void cli_wrap_hint (struct ui_out *uiout, char *identstring);
-static void cli_flush (struct ui_out *uiout);
-static int cli_redirect (struct ui_out *uiout, struct ui_file *outstream);
-
-/* This is the CLI ui-out implementation functions vector */
-
-/* FIXME: This can be initialized dynamically after default is set to
-   handle initial output in main.c */
-
-static struct ui_out_impl cli_ui_out_impl =
-{
-  cli_table_begin,
-  cli_table_body,
-  cli_table_end,
-  cli_table_header,
-  cli_begin,
-  cli_end,
-  cli_field_int,
-  cli_field_skip,
-  cli_field_string,
-  cli_field_fmt,
-  cli_spaces,
-  cli_text,
-  cli_message,
-  cli_wrap_hint,
-  cli_flush,
-  cli_redirect,
-  0, /* Does not need MI hacks (i.e. needs CLI hacks).  */
-};
 
 /* Prototypes for local functions */
 
-extern void _initialize_cli_out (void);
+static void cli_text (struct ui_out *uiout, const char *string);
 
 static void field_separator (void);
 
 static void out_field_fmt (struct ui_out *uiout, int fldno,
 			   const char *fldname,
-			   const char *format,...) ATTR_FORMAT (printf, 4, 5);
+			   const char *format,...) ATTRIBUTE_PRINTF (4, 5);
 
-/* local variables */
+/* The destructor.  */
 
-/* (none yet) */
+static void
+cli_uiout_dtor (struct ui_out *ui_out)
+{
+  cli_out_data *data = ui_out_data (ui_out);
+
+  VEC_free (ui_filep, data->streams);
+  xfree (data);
+}
+
+/* These are the CLI output functions */
 
 /* Mark beginning of a table */
 
-void
+static void
 cli_table_begin (struct ui_out *uiout, int nbrofcols,
 		 int nr_rows,
 		 const char *tblid)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (nr_rows == 0)
     data->suppress_output = 1;
   else
@@ -125,10 +70,11 @@ cli_table_begin (struct ui_out *uiout, int nbrofcols,
 
 /* Mark beginning of a table body */
 
-void
+static void
 cli_table_body (struct ui_out *uiout)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
   /* first, close the table header line */
@@ -137,84 +83,98 @@ cli_table_body (struct ui_out *uiout)
 
 /* Mark end of a table */
 
-void
+static void
 cli_table_end (struct ui_out *uiout)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   data->suppress_output = 0;
 }
 
 /* Specify table header */
 
-void
+static void
 cli_table_header (struct ui_out *uiout, int width, enum ui_align alignment,
 		  const char *col_name,
 		  const char *colhdr)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
-  cli_field_string (uiout, 0, width, alignment, 0, colhdr);
+
+  /* Always go through the function pointer (virtual function call).
+     We may have been extended.  */
+  uo_field_string (uiout, 0, width, alignment, 0, colhdr);
 }
 
 /* Mark beginning of a list */
 
-void
+static void
 cli_begin (struct ui_out *uiout,
 	   enum ui_out_type type,
 	   int level,
 	   const char *id)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
 }
 
 /* Mark end of a list */
 
-void
+static void
 cli_end (struct ui_out *uiout,
 	 enum ui_out_type type,
 	 int level)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
 }
 
 /* output an int field */
 
-void
+static void
 cli_field_int (struct ui_out *uiout, int fldno, int width,
 	       enum ui_align alignment,
 	       const char *fldname, int value)
 {
-  char buffer[20];		/* FIXME: how many chars long a %d can become? */
-
+  char buffer[20];	/* FIXME: how many chars long a %d can become? */
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
-  sprintf (buffer, "%d", value);
-  cli_field_string (uiout, fldno, width, alignment, fldname, buffer);
+  xsnprintf (buffer, sizeof (buffer), "%d", value);
+
+  /* Always go through the function pointer (virtual function call).
+     We may have been extended.  */
+  uo_field_string (uiout, fldno, width, alignment, fldname, buffer);
 }
 
 /* used to ommit a field */
 
-void
+static void
 cli_field_skip (struct ui_out *uiout, int fldno, int width,
 		enum ui_align alignment,
 		const char *fldname)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
-  cli_field_string (uiout, fldno, width, alignment, fldname, "");
+
+  /* Always go through the function pointer (virtual function call).
+     We may have been extended.  */
+  uo_field_string (uiout, fldno, width, alignment, fldname, "");
 }
 
 /* other specific cli_field_* end up here so alignment and field
    separators are both handled by cli_field_string */
 
-void
+static void
 cli_field_string (struct ui_out *uiout,
 		  int fldno,
 		  int width,
@@ -224,8 +184,8 @@ cli_field_string (struct ui_out *uiout,
 {
   int before = 0;
   int after = 0;
-
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
 
@@ -265,7 +225,7 @@ cli_field_string (struct ui_out *uiout,
 
 /* This is the only field function that does not align.  */
 
-void
+static void ATTRIBUTE_PRINTF (6, 0)
 cli_field_fmt (struct ui_out *uiout, int fldno,
 	       int width, enum ui_align align,
 	       const char *fldname,
@@ -273,74 +233,93 @@ cli_field_fmt (struct ui_out *uiout, int fldno,
 	       va_list args)
 {
   cli_out_data *data = ui_out_data (uiout);
+  struct ui_file *stream;
+
   if (data->suppress_output)
     return;
 
-  vfprintf_filtered (data->stream, format, args);
+  stream = VEC_last (ui_filep, data->streams);
+  vfprintf_filtered (stream, format, args);
 
   if (align != ui_noalign)
     field_separator ();
 }
 
-void
+static void
 cli_spaces (struct ui_out *uiout, int numspaces)
 {
   cli_out_data *data = ui_out_data (uiout);
+  struct ui_file *stream;
+
   if (data->suppress_output)
     return;
-  print_spaces_filtered (numspaces, data->stream);
+
+  stream = VEC_last (ui_filep, data->streams);
+  print_spaces_filtered (numspaces, stream);
 }
 
-void
+static void
 cli_text (struct ui_out *uiout, const char *string)
 {
   cli_out_data *data = ui_out_data (uiout);
+  struct ui_file *stream;
+
   if (data->suppress_output)
     return;
-  fputs_filtered (string, data->stream);
+
+  stream = VEC_last (ui_filep, data->streams);
+  fputs_filtered (string, stream);
 }
 
-void
+static void ATTRIBUTE_PRINTF (3, 0)
 cli_message (struct ui_out *uiout, int verbosity,
 	     const char *format, va_list args)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
+
   if (ui_out_get_verblvl (uiout) >= verbosity)
-    vfprintf_unfiltered (data->stream, format, args);
+    {
+      struct ui_file *stream = VEC_last (ui_filep, data->streams);
+
+      vfprintf_unfiltered (stream, format, args);
+    }
 }
 
-void
+static void
 cli_wrap_hint (struct ui_out *uiout, char *identstring)
 {
   cli_out_data *data = ui_out_data (uiout);
+
   if (data->suppress_output)
     return;
   wrap_here (identstring);
 }
 
-void
+static void
 cli_flush (struct ui_out *uiout)
 {
   cli_out_data *data = ui_out_data (uiout);
-  gdb_flush (data->stream);
+  struct ui_file *stream = VEC_last (ui_filep, data->streams);
+
+  gdb_flush (stream);
 }
 
-int
+/* OUTSTREAM as non-NULL will push OUTSTREAM on the stack of output streams
+   and make it therefore active.  OUTSTREAM as NULL will pop the last pushed
+   output stream; it is an internal error if it does not exist.  */
+
+static int
 cli_redirect (struct ui_out *uiout, struct ui_file *outstream)
 {
-  struct ui_out_data *data = ui_out_data (uiout);
+  cli_out_data *data = ui_out_data (uiout);
+
   if (outstream != NULL)
-    {
-      data->original_stream = data->stream;
-      data->stream = outstream;
-    }
-  else if (data->original_stream != NULL)
-    {
-      data->stream = data->original_stream;
-      data->original_stream = NULL;
-    }
+    VEC_safe_push (ui_filep, data->streams, outstream);
+  else
+    VEC_pop (ui_filep, data->streams);
 
   return 0;
 }
@@ -357,10 +336,11 @@ out_field_fmt (struct ui_out *uiout, int fldno,
 	       const char *format,...)
 {
   cli_out_data *data = ui_out_data (uiout);
+  struct ui_file *stream = VEC_last (ui_filep, data->streams);
   va_list args;
 
   va_start (args, format);
-  vfprintf_filtered (data->stream, format, args);
+  vfprintf_filtered (stream, format, args);
 
   va_end (args);
 }
@@ -370,21 +350,58 @@ out_field_fmt (struct ui_out *uiout, int fldno,
 static void
 field_separator (void)
 {
-  cli_out_data *data = ui_out_data (uiout);
-  fputc_filtered (' ', data->stream);
+  cli_out_data *data = ui_out_data (current_uiout);
+  struct ui_file *stream = VEC_last (ui_filep, data->streams);
+
+  fputc_filtered (' ', stream);
 }
 
-/* Initalize private members at startup.  */
+/* This is the CLI ui-out implementation functions vector */
+
+const struct ui_out_impl cli_ui_out_impl =
+{
+  cli_table_begin,
+  cli_table_body,
+  cli_table_end,
+  cli_table_header,
+  cli_begin,
+  cli_end,
+  cli_field_int,
+  cli_field_skip,
+  cli_field_string,
+  cli_field_fmt,
+  cli_spaces,
+  cli_text,
+  cli_message,
+  cli_wrap_hint,
+  cli_flush,
+  cli_redirect,
+  cli_uiout_dtor,
+  0, /* Does not need MI hacks (i.e. needs CLI hacks).  */
+};
+
+/* Constructor for a `cli_out_data' object.  */
+
+void
+cli_out_data_ctor (cli_out_data *self, struct ui_file *stream)
+{
+  gdb_assert (stream != NULL);
+
+  self->streams = NULL;
+  VEC_safe_push (ui_filep, self->streams, stream);
+
+  self->suppress_output = 0;
+}
+
+/* Initialize private members at startup.  */
 
 struct ui_out *
 cli_out_new (struct ui_file *stream)
 {
   int flags = ui_source_list;
+  cli_out_data *data = XNEW (cli_out_data);
 
-  cli_out_data *data = XMALLOC (cli_out_data);
-  data->stream = stream;
-  data->original_stream = NULL;
-  data->suppress_output = 0;
+  cli_out_data_ctor (data, stream);
   return ui_out_new (&cli_ui_out_impl, data, flags);
 }
 
@@ -392,14 +409,10 @@ struct ui_file *
 cli_out_set_stream (struct ui_out *uiout, struct ui_file *stream)
 {
   cli_out_data *data = ui_out_data (uiout);
-  struct ui_file *old = data->stream;
-  data->stream = stream;
+  struct ui_file *old;
+  
+  old = VEC_pop (ui_filep, data->streams);
+  VEC_quick_push (ui_filep, data->streams, stream);
+
   return old;
 }
-
-/* Standard gdb initialization hook.  */
-void
-_initialize_cli_out (void)
-{
-  /* nothing needs to be done */
-}
