@@ -1,6 +1,6 @@
 /* Manage register sets.
 
-   Copyright (C) 2003-2014 Free Software Foundation, Inc.
+   Copyright (C) 2003-2015 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -23,14 +23,6 @@
 struct gdbarch;
 struct regcache;
 
-/* Data structure for the supported register notes in a core file.  */
-struct core_regset_section
-{
-  const char *sect_name;
-  int size;
-  const char *human_name;
-};
-
 /* Data structure describing a register set.  */
 
 typedef void (supply_regset_ftype) (const struct regset *, struct regcache *,
@@ -41,9 +33,10 @@ typedef void (collect_regset_ftype) (const struct regset *,
 
 struct regset
 {
-  /* Data pointer for private use by the methods below, presumably
-     providing some sort of description of the register set.  */
-  const void *descr;
+  /* Pointer to a "register map", for private use by the methods
+     below.  Typically describes how the regset's registers are
+     arranged in the buffer collected to or supplied from.  */
+  const void *regmap;
 
   /* Function supplying values in a register set to a register cache.  */
   supply_regset_ftype *supply_regset;
