@@ -1,8 +1,6 @@
 /* Code dealing with register stack frames, for GDB, the GNU debugger.
 
-   Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
-   1996, 1997, 1998, 1999, 2000, 2001, 2002, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 1986-2015 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -48,21 +46,12 @@ sentinel_frame_prev_register (struct frame_info *this_frame,
 			      void **this_prologue_cache,
 			      int regnum)
 {
-  struct gdbarch *gdbarch = get_frame_arch (this_frame);
   struct frame_unwind_cache *cache = *this_prologue_cache;
   struct value *value;
 
-  /* Return the actual value.  */
-  value = allocate_value (register_type (gdbarch, regnum));
-  VALUE_LVAL (value) = lval_register;
-  VALUE_REGNUM (value) = regnum;
+  value = regcache_cooked_read_value (cache->regcache, regnum);
   VALUE_FRAME_ID (value) = get_frame_id (this_frame);
 
-  /* Use the regcache_cooked_read() method so that it, on the fly,
-     constructs either a raw or pseudo register from the raw
-     register cache.  */
-  regcache_cooked_read (cache->regcache, regnum, value_contents_raw (value));
-
   return value;
 }
 
@@ -86,9 +75,10 @@ sentinel_frame_prev_arch (struct frame_info *this_frame,
   return get_regcache_arch (cache->regcache);
 }
 
-const struct frame_unwind sentinel_frame_unwinder =
+const struct frame_unwind sentinel_frame_unwind =
 {
   SENTINEL_FRAME,
+  default_frame_unwind_stop_reason,
   sentinel_frame_this_id,
   sentinel_frame_prev_register,
   NULL,
@@ -96,5 +86,3 @@ const struct frame_unwind sentinel_frame_unwinder =
   NULL,
   sentinel_frame_prev_arch,
 };
-
-const struct frame_unwind *const sentinel_frame_unwind = &sentinel_frame_unwinder;
